// commands for sea_objects and heirs
// subsim (C)+(W) Thorsten Jordan. SEE LICENSE

#include "command.h"
#include "game.h"
#include "system.h"
#include "airplane.h"
#include "ship.h"
#include "submarine.h"


command* command::create(istream& in, class game& gm)
{
	unsigned type = read_u16(in);
	switch (type) {
		// sea_object
		case head_to_ang: return new command_head_to_ang(in, gm);
		case rudder_left: return new command_rudder_left(in, gm);
		case rudder_right: return new command_rudder_right(in, gm);
		case rudder_hard_left: return new command_rudder_hard_left(in, gm);
		case rudder_hard_right: return new command_rudder_hard_right(in, gm);
		case rudder_midships: return new command_rudder_midships(in, gm);
		case set_throttle: return new command_set_throttle(in, gm);
		// airplane
		case roll_left: return new command_roll_left(in, gm);
		case roll_right: return new command_roll_right(in, gm);
		case roll_zero: return new command_roll_zero(in, gm);
		case pitch_up: return new command_pitch_up(in, gm);
		case pitch_down: return new command_pitch_down(in, gm);
		case pitch_zero: return new command_pitch_zero(in, gm);
		// ship
		case fire_shell_at: return new command_fire_shell_at(in, gm);
		// submarine
		case scope_up: return new command_scope_up(in, gm);
		case scope_down: return new command_scope_down(in, gm);
		case scope_to: return new command_scope_to(in, gm);
		case snorkel_up: return new command_snorkel_up(in, gm);
		case snorkel_down: return new command_snorkel_down(in, gm);
		case planes_up: return new command_planes_up(in, gm);
		case planes_down: return new command_planes_down(in, gm);
		case planes_middle: return new command_planes_middle(in, gm);
		case dive_to_depth: return new command_dive_to_depth(in, gm);
		//case fire_torpedo:?
	};
	return 0;
}




void command_head_to_ang::exec(class game& gm) const
{
	myobj->head_to_ang(data, data2);
}

command_head_to_ang::command_head_to_ang(istream& in, class game& gm)
{
	myobj = gm.read_ship(in); data = read_double(in); data2 = read_bool(in);
}

void command_head_to_ang::save(ostream& out, const class game& gm) const
{
	write_u16(out, head_to_ang); gm.write(out, myobj); write_double(out, data.value()); write_bool(out, data2);
}

void command_rudder_left::exec(class game& gm) const
{
	myobj->rudder_left();
}

command_rudder_left::command_rudder_left(istream& in, class game& gm)
{
	myobj = gm.read_ship(in); 
}

void command_rudder_left::save(ostream& out, const class game& gm) const
{
	write_u16(out, rudder_left); gm.write(out, myobj);
}

void command_rudder_right::exec(class game& gm) const
{
	myobj->rudder_right();
}

command_rudder_right::command_rudder_right(istream& in, class game& gm)
{
	myobj = gm.read_ship(in); 
}

void command_rudder_right::save(ostream& out, const class game& gm) const
{
	write_u16(out, rudder_right); gm.write(out, myobj);
}

void command_rudder_hard_left::exec(class game& gm) const
{
	myobj->rudder_hard_left();
}

command_rudder_hard_left::command_rudder_hard_left(istream& in, class game& gm)
{
	myobj = gm.read_ship(in); 
}

void command_rudder_hard_left::save(ostream& out, const class game& gm) const
{
	write_u16(out, rudder_hard_left); gm.write(out, myobj);
}

void command_rudder_hard_right::exec(class game& gm) const
{
	myobj->rudder_hard_right();
}

command_rudder_hard_right::command_rudder_hard_right(istream& in, class game& gm)
{
	myobj = gm.read_ship(in); 
}

void command_rudder_hard_right::save(ostream& out, const class game& gm) const
{
	write_u16(out, rudder_hard_right); gm.write(out, myobj);
}

void command_rudder_midships::exec(class game& gm) const
{
	myobj->rudder_midships();
}

command_rudder_midships::command_rudder_midships(istream& in, class game& gm)
{
	myobj = gm.read_ship(in); 
}

void command_rudder_midships::save(ostream& out, const class game& gm) const
{
	write_u16(out, rudder_midships); gm.write(out, myobj);
}

void command_set_throttle::exec(class game& gm) const
{
	myobj->set_throttle(data);
}

command_set_throttle::command_set_throttle(istream& in, class game& gm)
{
	myobj = gm.read_ship(in); data = (ship::throttle_status)(read_i32(in));
}

void command_set_throttle::save(ostream& out, const class game& gm) const
{
	write_u16(out, set_throttle); gm.write(out, myobj); write_i32(out, int(data));
}

void command_roll_left::exec(class game& gm) const
{
	myobj->roll_left();
}

command_roll_left::command_roll_left(istream& in, class game& gm)
{
	myobj = gm.read_airplane(in); 
}

void command_roll_left::save(ostream& out, const class game& gm) const
{
	write_u16(out, roll_left); gm.write(out, myobj);
}

void command_roll_right::exec(class game& gm) const
{
	myobj->roll_right();
}

command_roll_right::command_roll_right(istream& in, class game& gm)
{
	myobj = gm.read_airplane(in); 
}

void command_roll_right::save(ostream& out, const class game& gm) const
{
	write_u16(out, roll_right); gm.write(out, myobj);
}

void command_roll_zero::exec(class game& gm) const
{
	myobj->roll_zero();
}

command_roll_zero::command_roll_zero(istream& in, class game& gm)
{
	myobj = gm.read_airplane(in); 
}

void command_roll_zero::save(ostream& out, const class game& gm) const
{
	write_u16(out, roll_zero); gm.write(out, myobj);
}

void command_pitch_up::exec(class game& gm) const
{
	myobj->pitch_up();
}

command_pitch_up::command_pitch_up(istream& in, class game& gm)
{
	myobj = gm.read_airplane(in); 
}

void command_pitch_up::save(ostream& out, const class game& gm) const
{
	write_u16(out, pitch_up); gm.write(out, myobj);
}

void command_pitch_down::exec(class game& gm) const
{
	myobj->pitch_down();
}

command_pitch_down::command_pitch_down(istream& in, class game& gm)
{
	myobj = gm.read_airplane(in); 
}

void command_pitch_down::save(ostream& out, const class game& gm) const
{
	write_u16(out, pitch_down); gm.write(out, myobj);
}

void command_pitch_zero::exec(class game& gm) const
{
	myobj->pitch_zero();
}

command_pitch_zero::command_pitch_zero(istream& in, class game& gm)
{
	myobj = gm.read_airplane(in); 
}

void command_pitch_zero::save(ostream& out, const class game& gm) const
{
	write_u16(out, pitch_zero); gm.write(out, myobj);
}

void command_fire_shell_at::exec(class game& gm) const
{
	myobj->fire_shell_at(data);
}

command_fire_shell_at::command_fire_shell_at(istream& in, class game& gm)
{
	myobj = gm.read_ship(in); data.x = read_double(in); data.y = read_double(in);
}

void command_fire_shell_at::save(ostream& out, const class game& gm) const
{
	write_u16(out, fire_shell_at); gm.write(out, myobj); write_double(out, data.x); write_double(out, data.y);
}

void command_scope_up::exec(class game& gm) const
{
	myobj->scope_up();
}

command_scope_up::command_scope_up(istream& in, class game& gm)
{
	myobj = gm.read_submarine(in); 
}

void command_scope_up::save(ostream& out, const class game& gm) const
{
	write_u16(out, scope_up); gm.write(out, myobj);
}

void command_scope_down::exec(class game& gm) const
{
	myobj->scope_down();
}

command_scope_down::command_scope_down(istream& in, class game& gm)
{
	myobj = gm.read_submarine(in); 
}

void command_scope_down::save(ostream& out, const class game& gm) const
{
	write_u16(out, scope_down); gm.write(out, myobj);
}

void command_scope_to::exec(class game& gm) const
{
	myobj->scope_to(data);
}

command_scope_to::command_scope_to(istream& in, class game& gm)
{
	myobj = gm.read_submarine(in); data = read_double(in);
}

void command_scope_to::save(ostream& out, const class game& gm) const
{
	write_u16(out, scope_to); gm.write(out, myobj); write_double(out, data);
}

void command_snorkel_up::exec(class game& gm) const
{
	myobj->snorkel_up();
}

command_snorkel_up::command_snorkel_up(istream& in, class game& gm)
{
	myobj = gm.read_submarine(in); 
}

void command_snorkel_up::save(ostream& out, const class game& gm) const
{
	write_u16(out, snorkel_up); gm.write(out, myobj);
}

void command_snorkel_down::exec(class game& gm) const
{
	myobj->snorkel_down();
}

command_snorkel_down::command_snorkel_down(istream& in, class game& gm)
{
	myobj = gm.read_submarine(in); 
}

void command_snorkel_down::save(ostream& out, const class game& gm) const
{
	write_u16(out, snorkel_down); gm.write(out, myobj);
}

void command_planes_up::exec(class game& gm) const
{
	myobj->planes_up(data);
}

command_planes_up::command_planes_up(istream& in, class game& gm)
{
	myobj = gm.read_submarine(in); data = read_double(in);
}

void command_planes_up::save(ostream& out, const class game& gm) const
{
	write_u16(out, planes_up); gm.write(out, myobj); write_double(out, data);
}

void command_planes_down::exec(class game& gm) const
{
	myobj->planes_down(data);
}

command_planes_down::command_planes_down(istream& in, class game& gm)
{
	myobj = gm.read_submarine(in); data = read_double(in);
}

void command_planes_down::save(ostream& out, const class game& gm) const
{
	write_u16(out, planes_down); gm.write(out, myobj); write_double(out, data);
}

void command_planes_middle::exec(class game& gm) const
{
	myobj->planes_middle();
}

command_planes_middle::command_planes_middle(istream& in, class game& gm)
{
	myobj = gm.read_submarine(in); 
}

void command_planes_middle::save(ostream& out, const class game& gm) const
{
	write_u16(out, planes_middle); gm.write(out, myobj);
}

void command_dive_to_depth::exec(class game& gm) const
{
	myobj->dive_to_depth(data);
}

command_dive_to_depth::command_dive_to_depth(istream& in, class game& gm)
{
	myobj = gm.read_submarine(in); data = read_u32(in);
}

void command_dive_to_depth::save(ostream& out, const class game& gm) const
{
	write_u16(out, dive_to_depth); gm.write(out, myobj); write_u32(out, data);
}

void command_transfer_torpedo::exec(class game& gm) const
{
	myobj->transfer_torpedo(data, data2);
}

command_transfer_torpedo::command_transfer_torpedo(istream& in, class game& gm)
{
	myobj = gm.read_submarine(in); data = read_u8(in); data2 = read_u8(in);
}

void command_transfer_torpedo::save(ostream& out, const class game& gm) const
{
	write_u16(out, transfer_torpedo); gm.write(out, myobj); write_u8(out, data); write_u8(out, data2);
}

void command_set_trp_primaryrange::exec(class game& gm) const
{
	myobj->set_trp_primaryrange(data);
}

command_set_trp_primaryrange::command_set_trp_primaryrange(istream& in, class game& gm)
{
	myobj = gm.read_submarine(in); data = read_u8(in);
}

void command_set_trp_primaryrange::save(ostream& out, const class game& gm) const
{
	write_u16(out, set_trp_primaryrange); gm.write(out, myobj); write_u8(out, data);
}

void command_set_trp_secondaryrange::exec(class game& gm) const
{
	myobj->set_trp_secondaryrange(data);
}

command_set_trp_secondaryrange::command_set_trp_secondaryrange(istream& in, class game& gm)
{
	myobj = gm.read_submarine(in); data = read_u8(in);
}

void command_set_trp_secondaryrange::save(ostream& out, const class game& gm) const
{
	write_u16(out, set_trp_secondaryrange); gm.write(out, myobj); write_u8(out, data);
}

void command_set_trp_initialturn::exec(class game& gm) const
{
	myobj->set_trp_initialturn(data);
}

command_set_trp_initialturn::command_set_trp_initialturn(istream& in, class game& gm)
{
	myobj = gm.read_submarine(in); data = read_u8(in);
}

void command_set_trp_initialturn::save(ostream& out, const class game& gm) const
{
	write_u16(out, set_trp_initialturn); gm.write(out, myobj); write_u8(out, data);
}

void command_set_trp_searchpattern::exec(class game& gm) const
{
	myobj->set_trp_searchpattern(data);
}

command_set_trp_searchpattern::command_set_trp_searchpattern(istream& in, class game& gm)
{
	myobj = gm.read_submarine(in); data = read_u8(in);
}

void command_set_trp_searchpattern::save(ostream& out, const class game& gm) const
{
	write_u16(out, set_trp_searchpattern); gm.write(out, myobj); write_u8(out, data);
}

void command_set_trp_addleadangle::exec(class game& gm) const
{
	myobj->set_trp_addleadangle(data);
}

command_set_trp_addleadangle::command_set_trp_addleadangle(istream& in, class game& gm)
{
	myobj = gm.read_submarine(in); data = read_double(in);
}

void command_set_trp_addleadangle::save(ostream& out, const class game& gm) const
{
	write_u16(out, set_trp_addleadangle); gm.write(out, myobj); write_double(out, data.value());
}

void command_launch_torpedo::exec(class game& gm) const
{
	myobj->launch_torpedo(gm, data, data2);
}

command_launch_torpedo::command_launch_torpedo(istream& in, class game& gm)
{
	myobj = gm.read_submarine(in); data = read_i8(in); data2 = gm.read_sea_object(in);
}

void command_launch_torpedo::save(ostream& out, const class game& gm) const
{
	write_u16(out, launch_torpedo); gm.write(out, myobj); write_i8(out, data); gm.write(out, data2);
}
