// commands for sea_objects and heirs
// subsim (C)+(W) Thorsten Jordan. SEE LICENSE

#ifndef COMMAND_H
#define COMMAND_H

#include "vector2.h"
#include "angle.h"
#include "binstream.h"
#include "sea_object.h"
#include "ship.h"
#include "airplane.h"
#include "submarine.h"
#include <iostream>
using namespace std;


//fixme: this whole thing is useless.
//sending commands for network play is not needed, just send the object states
//and we even do not have any network play...

class command {
protected:
	command() {}
	command(const command& other);
	command& operator= (const command& other);
	
	sea_object* myobj;
	
	command(sea_object* obj) : myobj(obj) {}
	
public:
	enum types {
		// sea_object (no commands)
		// airplane
		roll_left,
		roll_right,
		roll_zero,
		pitch_up,
		pitch_down,
		pitch_zero,
		// ship
		head_to_ang,
		rudder_left,
		rudder_right,
		rudder_hard_left,
		rudder_hard_right,
		rudder_midships,
		set_throttle,
		fire_shell_at,	// fixme: it does nothing! its code is in ai!
		// submarine
		scope_up,
		scope_down,
		scope_to,
		snorkel_up,
		snorkel_down,
		planes_up,
		planes_down,
		planes_middle,
		dive_to_depth,
		transfer_torpedo,
		set_trp_primaryrange,
		set_trp_secondaryrange,
		set_trp_initialturn,
		set_trp_searchpattern,
		set_trp_addleadangle,
		launch_torpedo
	};
		
	virtual ~command() {}
	virtual void exec(class game& gm) const = 0;
	virtual void save(ostream& out, const class game& gm) const = 0;
	static command* create(istream& in, class game& gm);
};



class command_airplane : public command
{
protected:
	command_airplane() {}
	airplane* myobj;
	command_airplane(airplane* obj) : myobj(obj) {}
public:
	virtual ~command_airplane() {}
};



class command_ship : public command
{
protected:
	command_ship() {}
	ship* myobj;
	command_ship(ship* obj) : myobj(obj) {}
public:
	virtual ~command_ship() {}
};



class command_submarine : public command
{
protected:
	command_submarine() {}
	submarine* myobj;
	command_submarine(submarine* obj) : myobj(obj) {}
public:
	virtual ~command_submarine() {}
};



class command_head_to_ang : public command_ship
{
protected:
	command_head_to_ang();
	command_head_to_ang(const command_head_to_ang& other);
	command_head_to_ang& operator= (const command_head_to_ang& other);
	
	angle data;
	bool data2;
public:
	command_head_to_ang(ship* obj, const angle& d, bool d2) : command_ship(obj), data(d), data2(d2) {}
	~command_head_to_ang() {}
	void exec(class game& gm) const;
	command_head_to_ang(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_rudder_left : public command_ship
{
protected:
	command_rudder_left();
	command_rudder_left(const command_rudder_left& other);
	command_rudder_left& operator= (const command_rudder_left& other);
	
public:
	command_rudder_left(ship* obj) : command_ship(obj) {}
	~command_rudder_left() {}
	void exec(class game& gm) const;
	command_rudder_left(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_rudder_right : public command_ship
{
protected:
	command_rudder_right();
	command_rudder_right(const command_rudder_right& other);
	command_rudder_right& operator= (const command_rudder_right& other);
	
public:
	command_rudder_right(ship* obj) : command_ship(obj) {}
	~command_rudder_right() {}
	void exec(class game& gm) const;
	command_rudder_right(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_rudder_hard_left : public command_ship
{
protected:
	command_rudder_hard_left();
	command_rudder_hard_left(const command_rudder_hard_left& other);
	command_rudder_hard_left& operator= (const command_rudder_hard_left& other);
	
public:
	command_rudder_hard_left(ship* obj) : command_ship(obj) {}
	~command_rudder_hard_left() {}
	void exec(class game& gm) const;
	command_rudder_hard_left(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_rudder_hard_right : public command_ship
{
protected:
	command_rudder_hard_right();
	command_rudder_hard_right(const command_rudder_hard_right& other);
	command_rudder_hard_right& operator= (const command_rudder_hard_right& other);
	
public:
	command_rudder_hard_right(ship* obj) : command_ship(obj) {}
	~command_rudder_hard_right() {}
	void exec(class game& gm) const;
	command_rudder_hard_right(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_rudder_midships : public command_ship
{
protected:
	command_rudder_midships();
	command_rudder_midships(const command_rudder_midships& other);
	command_rudder_midships& operator= (const command_rudder_midships& other);
	
public:
	command_rudder_midships(ship* obj) : command_ship(obj) {}
	~command_rudder_midships() {}
	void exec(class game& gm) const;
	command_rudder_midships(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_set_throttle : public command_ship
{
protected:
	command_set_throttle();
	command_set_throttle(const command_set_throttle& other);
	command_set_throttle& operator= (const command_set_throttle& other);
	
	ship::throttle_status data;
public:
	command_set_throttle(ship* obj, const ship::throttle_status& d) : command_ship(obj), data(d) {}
	~command_set_throttle() {}
	void exec(class game& gm) const;
	command_set_throttle(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_roll_left : public command_airplane
{
protected:
	command_roll_left();
	command_roll_left(const command_roll_left& other);
	command_roll_left& operator= (const command_roll_left& other);
	
public:
	command_roll_left(airplane* obj) : command_airplane(obj) {}
	~command_roll_left() {}
	void exec(class game& gm) const;
	command_roll_left(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_roll_right : public command_airplane
{
protected:
	command_roll_right();
	command_roll_right(const command_roll_right& other);
	command_roll_right& operator= (const command_roll_right& other);
	
public:
	command_roll_right(airplane* obj) : command_airplane(obj) {}
	~command_roll_right() {}
	void exec(class game& gm) const;
	command_roll_right(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_roll_zero : public command_airplane
{
protected:
	command_roll_zero();
	command_roll_zero(const command_roll_zero& other);
	command_roll_zero& operator= (const command_roll_zero& other);
	
public:
	command_roll_zero(class airplane* obj) : command_airplane(obj) {}
	~command_roll_zero() {}
	void exec(class game& gm) const;
	command_roll_zero(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_pitch_up : public command_airplane
{
protected:
	command_pitch_up();
	command_pitch_up(const command_pitch_up& other);
	command_pitch_up& operator= (const command_pitch_up& other);
	
public:
	command_pitch_up(class airplane* obj) : command_airplane(obj) {}
	~command_pitch_up() {}
	void exec(class game& gm) const;
	command_pitch_up(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_pitch_down : public command_airplane
{
protected:
	command_pitch_down();
	command_pitch_down(const command_pitch_down& other);
	command_pitch_down& operator= (const command_pitch_down& other);
	
public:
	command_pitch_down(class airplane* obj) : command_airplane(obj) {}
	~command_pitch_down() {}
	void exec(class game& gm) const;
	command_pitch_down(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_pitch_zero : public command_airplane
{
protected:
	command_pitch_zero();
	command_pitch_zero(const command_pitch_zero& other);
	command_pitch_zero& operator= (const command_pitch_zero& other);
	
public:
	command_pitch_zero(class airplane* obj) : command_airplane(obj) {}
	~command_pitch_zero() {}
	void exec(class game& gm) const;
	command_pitch_zero(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_fire_shell_at : public command_ship
{
protected:
	command_fire_shell_at();
	command_fire_shell_at(const command_fire_shell_at& other);
	command_fire_shell_at& operator= (const command_fire_shell_at& other);
	
	vector2 data;
public:
	command_fire_shell_at(ship* obj, const vector2& d) : command_ship(obj), data(d) {}
	~command_fire_shell_at() {}
	void exec(class game& gm) const;
	command_fire_shell_at(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_scope_up : public command_submarine
{
protected:
	command_scope_up();
	command_scope_up(const command_scope_up& other);
	command_scope_up& operator= (const command_scope_up& other);
	
public:
	command_scope_up(submarine* obj) : command_submarine(obj) {}
	~command_scope_up() {}
	void exec(class game& gm) const;
	command_scope_up(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_scope_down : public command_submarine
{
protected:
	command_scope_down();
	command_scope_down(const command_scope_down& other);
	command_scope_down& operator= (const command_scope_down& other);
	
public:
	command_scope_down(submarine* obj) : command_submarine(obj) {}
	~command_scope_down() {}
	void exec(class game& gm) const;
	command_scope_down(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_scope_to : public command_submarine
{
protected:
	command_scope_to();
	command_scope_to(const command_scope_to& other);
	command_scope_to& operator= (const command_scope_to& other);
	
	double data;
public:
	command_scope_to(submarine* obj, const double& d) : command_submarine(obj), data(d) {}
	~command_scope_to() {}
	void exec(class game& gm) const;
	command_scope_to(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_snorkel_up : public command_submarine
{
protected:
	command_snorkel_up();
	command_snorkel_up(const command_snorkel_up& other);
	command_snorkel_up& operator= (const command_snorkel_up& other);
	
public:
	command_snorkel_up(submarine* obj) : command_submarine(obj) {}
	~command_snorkel_up() {}
	void exec(class game& gm) const;
	command_snorkel_up(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_snorkel_down : public command_submarine
{
protected:
	command_snorkel_down();
	command_snorkel_down(const command_snorkel_down& other);
	command_snorkel_down& operator= (const command_snorkel_down& other);
	
public:
	command_snorkel_down(submarine* obj) : command_submarine(obj) {}
	~command_snorkel_down() {}
	void exec(class game& gm) const;
	command_snorkel_down(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_planes_up : public command_submarine
{
protected:
	command_planes_up();
	command_planes_up(const command_planes_up& other);
	command_planes_up& operator= (const command_planes_up& other);
	
	double data;
public:
	command_planes_up(submarine* obj, const double& d) : command_submarine(obj), data(d) {}
	~command_planes_up() {}
	void exec(class game& gm) const;
	command_planes_up(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_planes_down : public command_submarine
{
protected:
	command_planes_down();
	command_planes_down(const command_planes_down& other);
	command_planes_down& operator= (const command_planes_down& other);
	
	double data;
public:
	command_planes_down(submarine* obj, const double& d) : command_submarine(obj), data(d) {}
	~command_planes_down() {}
	void exec(class game& gm) const;
	command_planes_down(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_planes_middle : public command_submarine
{
protected:
	command_planes_middle();
	command_planes_middle(const command_planes_middle& other);
	command_planes_middle& operator= (const command_planes_middle& other);
	
public:
	command_planes_middle(submarine* obj) : command_submarine(obj) {}
	~command_planes_middle() {}
	void exec(class game& gm) const;
	command_planes_middle(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_dive_to_depth : public command_submarine
{
protected:
	command_dive_to_depth();
	command_dive_to_depth(const command_dive_to_depth& other);
	command_dive_to_depth& operator= (const command_dive_to_depth& other);
	
	unsigned data;
public:
	command_dive_to_depth(submarine* obj, unsigned d) : command_submarine(obj), data(d) {}
	~command_dive_to_depth() {}
	void exec(class game& gm) const;
	command_dive_to_depth(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_transfer_torpedo : public command_submarine
{
protected:
	command_transfer_torpedo();
	command_transfer_torpedo(const command_transfer_torpedo& other);
	command_transfer_torpedo& operator= (const command_transfer_torpedo& other);
	
	unsigned data, data2;
public:
	command_transfer_torpedo(submarine* obj, unsigned d, unsigned d2) : command_submarine(obj), data(d), data2(d2) {}
	~command_transfer_torpedo() {}
	void exec(class game& gm) const;
	command_transfer_torpedo(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_set_trp_primaryrange : public command_submarine
{
protected:
	command_set_trp_primaryrange();
	command_set_trp_primaryrange(const command_set_trp_primaryrange& other);
	command_set_trp_primaryrange& operator= (const command_set_trp_primaryrange& other);
	
	unsigned data;
public:
	command_set_trp_primaryrange(submarine* obj, unsigned d) : command_submarine(obj), data(d) {}
	~command_set_trp_primaryrange() {}
	void exec(class game& gm) const;
	command_set_trp_primaryrange(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_set_trp_secondaryrange : public command_submarine
{
protected:
	command_set_trp_secondaryrange();
	command_set_trp_secondaryrange(const command_set_trp_secondaryrange& other);
	command_set_trp_secondaryrange& operator= (const command_set_trp_secondaryrange& other);
	
	unsigned data;
public:
	command_set_trp_secondaryrange(submarine* obj, unsigned d) : command_submarine(obj), data(d) {}
	~command_set_trp_secondaryrange() {}
	void exec(class game& gm) const;
	command_set_trp_secondaryrange(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_set_trp_initialturn : public command_submarine
{
protected:
	command_set_trp_initialturn();
	command_set_trp_initialturn(const command_set_trp_initialturn& other);
	command_set_trp_initialturn& operator= (const command_set_trp_initialturn& other);
	
	unsigned data;
public:
	command_set_trp_initialturn(submarine* obj, unsigned d) : command_submarine(obj), data(d) {}
	~command_set_trp_initialturn() {}
	void exec(class game& gm) const;
	command_set_trp_initialturn(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_set_trp_searchpattern : public command_submarine
{
protected:
	command_set_trp_searchpattern();
	command_set_trp_searchpattern(const command_set_trp_searchpattern& other);
	command_set_trp_searchpattern& operator= (const command_set_trp_searchpattern& other);
	
	unsigned data;
public:
	command_set_trp_searchpattern(submarine* obj, unsigned d) : command_submarine(obj), data(d) {}
	~command_set_trp_searchpattern() {}
	void exec(class game& gm) const;
	command_set_trp_searchpattern(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_set_trp_addleadangle : public command_submarine
{
protected:
	command_set_trp_addleadangle();
	command_set_trp_addleadangle(const command_set_trp_addleadangle& other);
	command_set_trp_addleadangle& operator= (const command_set_trp_addleadangle& other);
	
	angle data;
public:
	command_set_trp_addleadangle(submarine* obj, angle d) : command_submarine(obj), data(d) {}
	~command_set_trp_addleadangle() {}
	void exec(class game& gm) const;
	command_set_trp_addleadangle(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};

class command_launch_torpedo : public command_submarine
{
protected:
	command_launch_torpedo();
	command_launch_torpedo(const command_launch_torpedo& other);
	command_launch_torpedo& operator= (const command_launch_torpedo& other);
	
	int data;
	class sea_object* data2;
public:
	command_launch_torpedo(submarine* obj, int d, class sea_object* d2) : command_submarine(obj), data(d), data2(d2) {}
	~command_launch_torpedo() {}
	void exec(class game& gm) const;
	command_launch_torpedo(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};


// template
/*
class command_@ : public command
{
protected:
	command_@();
	command_@(const command_@& other);
	command_@& operator= (const command_@& other);
	
	class #* myobj;
	^ data;
public:
	command_@(class #* obj, const ^& d) : myobj(obj), data(d) {}
	~command_@() {}
	void exec(class game& gm) const;
	command_@(istream& in, class game& gm);
	void save(ostream& out, const class game& gm) const;
};
*/

#endif
