// sound
// subsim (C)+(W) Markus Petermann. SEE LICENSE

#ifndef _SOUND_H_
#define _SOUND_H_

class sound
{
public:
	enum { number_of_samples = 10 };

	struct sample
	{
		Uint8*	data;
		Uint32	dpos;
		Uint32	dlen;
		Uint8	dvolume;
	};

private:
	static struct sample sounds[number_of_samples];
	Uint8* sound_data;
	Uint32 sound_data_len;

	void init ();

protected:
	/** SDL_AudioSpec object with informations about the found audio device. */
	static SDL_AudioSpec audio_spec;
	static SDL_AudioSpec* audio_spec_ptr;
	/** Opens the audio device and sets the audio_spec attribute. */
	static void open_audio_device ();
	/** Closes audio device. This method is normally never called. */
	static void close_audio_device ();
	virtual void load_sound ( const string& filename );
	
	static bool initialized;	// has SDL been initialized?

public:
	static bool use_sound;	// set this to enable/disable sound

	/** Constructor. */
	sound ();
	/**
		Constructor.
		@param filename Name of WAV file.
	*/
	sound ( const string& filename );
	/** Destructor. */
	virtual ~sound ();
	/**
		Plays the specified sound with the specified volume.
		@param volume A float value between 0 and 1. Default value is 1.
	*/
	virtual void play ( double volume = 1.0f );

	/** This is the callback function needed by SDL. */
	static void mix_audio ( void* unused, Uint8* stream, int len );
};

#endif /* _SOUND_H_ */
